/*
 * gsampler-client-connection.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
	 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
	 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <string.h>
#include <glib/gi18n-lib.h>

#include "gsampler-client-connection.h"
#include "gsampler-option.h"
#include "gsampler-utils.h"

static gchar* 
gsampler_client_connection_read (GsamplerConnnection *ctx)
{
	char buf[256];
	GError *error = NULL;
	g_return_val_if_fail (ctx != NULL, NULL);

	g_socket_receive(ctx->sock, buf, 256, NULL, &error);
	if (error != NULL)
	{
		g_warning ("file %s: line %d: error: %s (%s, %d)", __FILE__, __LINE__, 
		           error->message, g_quark_to_string (error->domain), error->code);
		g_error_free (error);
		return NULL;
	}
	return g_strdup((gchar*) buf);
}

static gboolean
gsampler_client_connection_write (GsamplerConnnection *ctx, const gchar *mes)
{
	GError *error = NULL;
	g_return_val_if_fail (ctx != NULL, FALSE);
	g_return_val_if_fail (mes != NULL, FALSE);

	g_socket_send (ctx->sock, mes, strlen (mes), NULL, &error);
	if (error != NULL)
	{
		g_warning ("file %s: line %d: error: %s (%s, %d)", __FILE__, __LINE__, 
		           error->message, g_quark_to_string (error->domain), error->code);
		g_error_free (error);
		return FALSE;
	}
	return TRUE;
}

static gboolean
gsampler_client_start_serveur_event (GsamplerClient* self, const gchar* event) 
{
	gchar *result = NULL;
	
	if (gsampler_client_connection_write (self->priv->ctx1, event))
	{
		while (!g_socket_condition_wait (self->priv->ctx1->sock, G_IO_IN, NULL, NULL))
		{
			g_usleep (500000);
		}
		result = gsampler_client_connection_read (self->priv->ctx1);
		if (!string_contains (result, "OK")) 
		{
			g_warning ("Start Event Error : %s", result);
			g_free (result);
			return FALSE;
		}
	} else
	{
		g_warning ("Can't send the com : %s", event);
		return FALSE;
	}
	return TRUE;
}

static gboolean 
gsampler_client_connect_seveur_event (GsamplerClient* self) 
{
	g_return_val_if_fail (self != NULL, FALSE);

	if (!gsampler_client_start_serveur_event (self, "SET ECHO 0\n")) 
	{
		g_warning ("Can't set echo 0 to Linuxsampler");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE AUDIO_OUTPUT_DEVICE_COUNT\n")) 
	{
		g_warning ("Can't subscribe event : AUDIO_OUTPUT_DEVICE_COUNT");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE AUDIO_OUTPUT_DEVICE_INFO\n")) 
	{
		g_warning ("Can't subscribe event : AUDIO_OUTPUT_DEVICE_INFO");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE MIDI_INPUT_DEVICE_COUNT\n")) 
	{
		g_warning ("Can't subscribe event : MIDI_INPUT_DEVICE_COUNT");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE MIDI_INPUT_DEVICE_INFO\n")) 
	{
		g_warning ("Can't subscribe event : MIDI_INPUT_DEVICE_INFO");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE CHANNEL_COUNT\n")) 
	{
		g_warning ("Can't subscribe event : CHANNEL_COUNT");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE CHANNEL_INFO\n")) 
	{
		g_warning ("Can't subscribe event : CHANNEL_INFO\n");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE MIDI_INSTRUMENT_MAP_COUNT\n")) 
	{
		g_warning ("Can't subscribe event : MIDI_INSTRUMENT_MAP_COUNT");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE MIDI_INSTRUMENT_MAP_INFO\n")) 
	{
		g_warning ("Can't subscribe event : MIDI_INSTRUMENT_MAP_INFO");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE MIDI_INSTRUMENT_COUNT\n")) 
	{
		g_warning ("Can't subscribe event : MIDI_INSTRUMENT_COUNT");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE MIDI_INSTRUMENT_INFO\n")) 
	{
		g_warning ("Can't subscribe event : MIDI_INSTRUMENT_INFO");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE DB_INSTRUMENT_DIRECTORY_COUNT\n")) 
	{
		g_warning ("Can't subscribe event : DB_INSTRUMENT_DIRECTORY_COUNT");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE TOTAL_STREAM_COUNT\n")) 
	{
		g_warning ("Can't subscribe event : TOTAL_STREAM_COUNT");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE TOTAL_VOICE_COUNT\n")) 
	{
		g_warning ("Can't subscribe event : TOTAL_VOICE_COUNT");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE VOICE_COUNT\n")) 
	{
		g_warning ("Can't subscribe event : VOICE_COUNT\n");
		return FALSE;
	}
	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE STREAM_COUNT\n")) 
	{
		g_warning ("Can't subscribe event : STREAM_COUNT\n");
		return FALSE;
	}

	if (!gsampler_client_start_serveur_event (self, "SUBSCRIBE MISCELLANEOUS\n")) 
	{
		g_warning ("Can't subscribe event : MISCELLANEOUS\n");
		return FALSE;
	}
	return TRUE;
}

static gboolean
gsampler_client_connection_connect (GsamplerConnnection *ctx)
{
	GInetAddress *inet_addr = NULL;
	GSocketAddress *sock_addr = NULL;
	GError *error = NULL;
	g_return_val_if_fail (ctx != NULL, FALSE);

	ctx->sock = g_socket_new (G_SOCKET_FAMILY_IPV4, G_SOCKET_TYPE_STREAM, G_SOCKET_PROTOCOL_TCP, &error);
	if (error != NULL)
	{
		g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
		g_error_free (error);
		return FALSE;
	}
#ifdef G_OS_UNIX
	inet_addr = g_inet_address_new_from_string (ctx->host);
#else
	inet_addr = g_inet_address_new_loopback (G_SOCKET_FAMILY_IPV4);
#endif
	sock_addr = g_inet_socket_address_new (inet_addr, ctx->port);
	//g_socket_set_keepalive (ctx->sock, TRUE);
	//g_socket_set_blocking (ctx->sock, FALSE);
	g_socket_connect (ctx->sock, sock_addr, NULL, &error);
	if (error != NULL)
	{
		g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
		g_error_free (error);
		return FALSE;
	}
	if (!g_socket_check_connect_result (ctx->sock, &error)) 
	{
		if (error != NULL)
		{
			g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
			g_error_free (error);
			
		}
		return FALSE;
	} else {
		if (g_socket_is_connected (ctx->sock))
		{
			ctx->state = GSAMPLER_CONNECTED;
			g_socket_set_timeout (ctx->sock, 3);
		} else {
			ctx->state = GSAMPLER_CTX_ERROR;
		}
	}
	return TRUE;
}

/*static gboolean
gsampler_client_connection_connect (GsamplerConnnection *ctx)
{
	GSocketClient *client = NULL;
	GError *error = NULL;
	g_return_val_if_fail (ctx != NULL, FALSE);

	client = g_socket_client_new();
#ifdef G_OS_UNIX	
	GInetAddress *inet_addr = g_inet_address_new_from_string (ctx->host);
#else
	GInetAddress *inet_addr = g_inet_address_new_loopback (G_SOCKET_FAMILY_IPV4);
#endif
	GSocketAddress *sock_addr = g_inet_socket_address_new (inet_addr, ctx->port);
	ctx->conn = g_socket_client_connect (client, (GSocketConnectable*) sock_addr,
	                                     NULL, &error);
	if (error != NULL)
	{
		g_error ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
		g_error_free (error);
		g_object_unref (client);
		return FALSE;
	}
	g_object_unref (client);
	if (ctx->conn) 
	{
		ctx->sock = g_socket_connection_get_socket(ctx->conn);
		if (g_socket_is_connected (ctx->sock))
		{
			ctx->sockfd = g_socket_get_fd(ctx->sock);
			ctx->state = GSAMPLER_CONNECTED;
			return TRUE;
		} 
		else 
		{
			return FALSE;
		}
	}
	else
	{
		return FALSE;
	}
}*/

static gboolean
gsampler_client_connection_disconnect (GsamplerConnnection *ctx)
{
	GError *error = NULL;
	g_return_val_if_fail (ctx != NULL, FALSE);

	if (ctx->sock != NULL && g_socket_is_connected (ctx->sock))
	{
		g_socket_close (ctx->sock, &error);
		if (error != NULL)
		{
			g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
			g_error_free (error);
			return FALSE;
		}
		g_object_unref (ctx->sock);
	}
	g_free (ctx->host);
	ctx->port = 0;
	return TRUE;
}

static gboolean 
gsampler_client_callback (GIOChannel* source, GIOCondition condition, gpointer data) 
{
	gchar* buf = NULL;
	gsize bytes_read = 0;
	GError * error = NULL;
	gboolean result = FALSE;
	GsamplerClient *self = (GsamplerClient*) data;
	g_return_val_if_fail (GSAMPLER_IS_CLIENT (self), FALSE);
	g_return_val_if_fail (source != NULL, FALSE);

	//gdk_threads_enter ();
	if (!g_source_is_destroyed (g_main_current_source ()))
	{
		GIOStatus stat = G_IO_STATUS_NORMAL;
		switch (condition) 
		{
			case G_IO_IN:
				stat = g_io_channel_read_line (source, &buf, &bytes_read, NULL, &error);
				if (error != NULL) 
				{
					g_warning ("file: %s, line: %d, error: %s", __FILE__, __LINE__, error->message);
					g_free (buf);
					g_error_free (error);
				}
				else if (stat != G_IO_STATUS_NORMAL)
				{
					g_free (buf);
					g_warning (_("Can't read chars"));
				}
				else if (bytes_read != 0) 
				{
					g_signal_emit_by_name (self, "client-event", g_strdup (buf));
					result = TRUE;
				}
				g_free (buf);
				break;
			default:
				g_warning ("file %s: line %d: watch de merde", __FILE__, __LINE__);
				break;
		}
	}
	//gdk_threads_leave ();
	return result;
}

gboolean 
gsampler_client_connection_start (GsamplerClient* self) {
	GsamplerOption* option = NULL;
	gint socket_fd = 0;
	g_return_val_if_fail (self != NULL, FALSE);

	option = gsampler_option_get_instance ();
	self->priv->ctx1->host = gsampler_option_get_string (option, "Serveur", "serveur-host");
	self->priv->ctx2->host = g_strdup (self->priv->ctx1->host);
	self->priv->ctx1->port = gsampler_option_get_integer (option, "Serveur", "serveur-port");
	self->priv->ctx2->port = self->priv->ctx1->port;
	self->priv->timeout = 1000 * gsampler_option_get_integer (option, "Serveur", "serveur-timeout");
	g_object_unref (option);

	if ((gsampler_client_connection_connect (self->priv->ctx1)) &&
	    (gsampler_client_connection_connect (self->priv->ctx2))) 
	{
		if (!gsampler_client_connect_seveur_event (self)) 
		{
			g_warning ("Can't connect to the event");
			return FALSE;
		}
		socket_fd = g_socket_get_fd(self->priv->ctx1->sock);
#ifdef G_OS_UNIX
		self->priv->chan = g_io_channel_unix_new (socket_fd);
#else
		self->priv->chan = g_io_channel_win32_new_socket (socket_fd);
#endif
		if (self->priv->chan == NULL)
		{
			g_warning ("file %s: line %d: Can' create the giochannel", __FILE__, __LINE__);
			return FALSE;
		}
		g_io_channel_set_close_on_unref (self->priv->chan, TRUE);
		//g_io_channel_set_buffered (self->priv->chan, FALSE);
		self->priv->chan_watch_id = g_io_add_watch (self->priv->chan, G_IO_IN | G_IO_HUP,
			                                        gsampler_client_callback, self);
		if (self->priv->chan_watch_id == 0)
		{
			g_warning ("Can't add gio watch");
			return FALSE;
		} else 
		{
			g_signal_connect (self, "client-event", (GCallback) gsampler_client_callback_event, NULL);
		}
	} else
	{
		g_warning ("Can't connect the client");
		return FALSE;
	}
	self->priv->verrou = FALSE;
	return TRUE;
}

gboolean 
gsampler_client_connection_stop (GsamplerClient* self) 
{
	//GError * error = NULL;
	g_return_val_if_fail (self != NULL, FALSE);

	self->priv->verrou = TRUE;
	/*if (self->priv->chan != NULL)
	{
		GIOStatus status = g_io_channel_shutdown (self->priv->chan, FALSE, NULL);
		if (status == G_IO_STATUS_NORMAL) 
		{
			g_io_channel_unref (self->priv->chan);
		} else
		{
			g_warning (_("Can't  shutdown the channel status %d"), status);
		}
		g_source_remove (self->priv->chan_watch_id);
	}*/
	if (self->priv->chan_watch_id != 0)
		g_source_remove (self->priv->chan_watch_id);
	if (!gsampler_client_connection_disconnect (self->priv->ctx1))
		return FALSE;
	if (!gsampler_client_connection_disconnect (self->priv->ctx2))
		return FALSE;
	return TRUE;
}