/*
 * gsampler-client-base.h
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _GSAMPLER_CLIENT_H_
#define _GSAMPLER_CLIENT_H_

#include <glib.h>
#include <gio/gio.h>

G_BEGIN_DECLS

enum {
	GSAMPLER_DISCONNECTED, 
	GSAMPLER_CONNECTING, 
	GSAMPLER_CONNECTED,
	GSAMPLER_CTX_ERROR,
	GSAMPLER_COM_ERROR
};

typedef struct _GsamplerConnnection GsamplerConnnection;
struct _GsamplerConnnection {
	GSocket *sock;
	GSocketConnection* conn;
	gint sockfd;
	gchar* host;
	gint port;
	gint state;
};

typedef struct _GsamplerServer GsamplerServer;
struct _GsamplerServer {
	gint started;
	GPid pid;
	gint watch_id;
	GIOChannel* server_chan;
};

typedef struct _GsamplerClientJob GsamplerClientJob;
struct _GsamplerClientJob {
	gpointer owner;
	gchar* command;
};

#define GSAMPLER_TYPE_CLIENT (gsampler_client_get_type ())
#define GSAMPLER_CLIENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GSAMPLER_TYPE_CLIENT, GsamplerClient))
#define GSAMPLER_CLIENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GSAMPLER_TYPE_CLIENT, GsamplerClientClass))
#define GSAMPLER_IS_CLIENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GSAMPLER_TYPE_CLIENT))
#define GSAMPLER_IS_CLIENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GSAMPLER_TYPE_CLIENT))
#define GSAMPLER_CLIENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GSAMPLER_TYPE_CLIENT, GsamplerClientClass))
#define GSAMPLER_CLIENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSAMPLER_TYPE_CLIENT, GsamplerClientPrivate))

typedef struct _GsamplerClient GsamplerClient;
typedef struct _GsamplerClientClass GsamplerClientClass;
typedef struct _GsamplerClientPrivate GsamplerClientPrivate;

struct _GsamplerClient {
	GObject parent_instance;
	GsamplerClientPrivate * priv;
};

struct _GsamplerClientClass {
	GObjectClass parent_class;
};

struct _GsamplerClientPrivate {
	GIOChannel* chan;
	gint timeout;
	GsamplerConnnection* ctx1;
	GsamplerConnnection* ctx2;
	GsamplerServer* server;
	gboolean verrou;
	gint chan_watch_id;
};

/* Client */
GType 			gsampler_client_get_type (void) G_GNUC_CONST;
GType 			gsampler_db_job_info_get_type (void) G_GNUC_CONST;

GsamplerClient* gsampler_client_new (void);
GsamplerClient* gsampler_client_get_instance (void);

void    		gsampler_client_callback_event (GsamplerClient *self, const gchar* mes, gpointer data);
gboolean 		gsampler_client_start (GsamplerClient* client);
gboolean		gsampler_client_stop (GsamplerClient* client, gboolean force);
gboolean		gsampler_client_reset_client (GsamplerClient* client);
gboolean		gsampler_client_restart (GsamplerClient* client);
void 			gsampler_client_client_timeout (GsamplerClient* self);
gchar* 			gsampler_client_send_commande (GsamplerClient* self, const gchar* mes);
gchar* 			gsampler_client_get_master_volume (GsamplerClient* self);
gboolean 		gsampler_client_set_master_volume (GsamplerClient* self, gdouble vol);
gboolean 		gsampler_client_client_reset (GsamplerClient* self);
gint			gsampler_client_get_int (GsamplerClient* self, const gchar* com);

GSList*			gsampler_client_client_get_info (GsamplerClient* self, const gchar* mes);
gboolean		gsampler_client_send_bool (GsamplerClient* self, const gchar* com);
gint			gsampler_client_create (GsamplerClient* self, const gchar* com);
gchar*			gsampler_client_get_char (GsamplerClient* self, const gchar* com);
gchar**			gsampler_client_get_list (GsamplerClient* self, const gchar* com);

G_END_DECLS

#endif /* _GSAMPLER_CLIENT_H_ */